@@ -35,25 +35,27 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.jdt.core.IAnnotatable;
 import org.eclipse.jdt.core.IAnnotation;
 import org.eclipse.jdt.core.IField;
+import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IMethod;
 import org.eclipse.jdt.core.IType;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
 import org.eclipse.jdt.core.dom.SimpleName;
+import org.eclipse.jdt.core.dom.Type;
 import org.eclipse.jdt.core.search.SearchMatch;
+import org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
 import org.eclipse.jdt.internal.compiler.ast.Annotation;
-import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
 import org.eclipse.jdt.internal.compiler.ast.Expression;
 import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
 import org.eclipse.jdt.internal.compiler.ast.ForeachStatement;
 import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
 import org.eclipse.jdt.internal.compiler.ast.MessageSend;
+import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
 import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
 import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
 import org.eclipse.jdt.internal.compiler.lookup.Scope;
 import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
-import org.eclipse.jdt.internal.compiler.parser.Parser;
 import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 import org.eclipse.jdt.internal.core.SourceField;
 import org.eclipse.jdt.internal.core.dom.rewrite.NodeRewriteEvent;
@@ -62,7 +64,7 @@ import org.eclipse.jdt.internal.core.dom.rewrite.TokenScanner;
 import org.eclipse.jdt.internal.corext.refactoring.SearchResultGroup;
 import org.eclipse.jdt.internal.corext.refactoring.structure.ASTNodeSearchUtil;
 
-import lombok.eclipse.EclipseAugments;
+import static lombok.eclipse.EcjAugments.ASTNode_generatedBy;
 
 /** These contain a mix of the following:
  * <ul>
@@ -83,20 +85,24 @@ final class PatchFixesHider {
 	public static final class Util {
 		private static ClassLoader shadowLoader;
 		
+		public static ClassLoader getShadowLoader() {
+			if (shadowLoader == null) {
+				try {
+					Class.forName("lombok.core.LombokNode");
+					// If we get here, then lombok is already available.
+					shadowLoader = Util.class.getClassLoader();
+				} catch (ClassNotFoundException e) {
+					// If we get here, it isn't, and we should use the shadowloader.
+					shadowLoader = Main.getShadowClassLoader();
+				}
+			}
+			
+			return shadowLoader;
+		}
+		
 		public static Class<?> shadowLoadClass(String name) {
 			try {
-				if (shadowLoader == null) {
-					try {
-						Class.forName("lombok.core.LombokNode");
-						// If we get here, then lombok is already available.
-						shadowLoader = Util.class.getClassLoader();
-					} catch (ClassNotFoundException e) {
-						// If we get here, it isn't, and we should use the shadowloader.
-						shadowLoader = Main.getShadowClassLoader();
-					}
-				}
-				
-				return Class.forName(name, true, shadowLoader);
+				return Class.forName(name, true, getShadowLoader());
 			} catch (ClassNotFoundException e) {
 				throw sneakyThrow(e);
 			}
@@ -110,6 +116,11 @@ final class PatchFixesHider {
 			}
 		}
 		
+		public static Method findMethodAnyArgs(Class<?> type, String name) {
+			for (Method m : type.getDeclaredMethods()) if (name.equals(m.getName())) return m;
+			throw sneakyThrow(new NoSuchMethodException(type.getName() + "::" + name));
+		}
+		
 		public static Object invokeMethod(Method method, Object... args) {
 			try {
 				return method.invoke(null, args);
@@ -169,20 +180,26 @@ final class PatchFixesHider {
 	}
 	
 	public static final class Transform {
-		private static final Method TRANSFORM;
-		private static final Method TRANSFORM_SWAPPED;
+		private static Method TRANSFORM;
+		private static Method TRANSFORM_SWAPPED;
 		
-		static {
+		private static synchronized void init(ClassLoader prepend) {
+			if (TRANSFORM != null) return;
+			
+			Main.prependClassLoader(prepend);
 			Class<?> shadowed = Util.shadowLoadClass("lombok.eclipse.TransformEclipseAST");
-			TRANSFORM = Util.findMethod(shadowed, "transform", Parser.class, CompilationUnitDeclaration.class);
-			TRANSFORM_SWAPPED = Util.findMethod(shadowed, "transform_swapped", CompilationUnitDeclaration.class, Parser.class);
+			TRANSFORM = Util.findMethodAnyArgs(shadowed, "transform");
+			TRANSFORM_SWAPPED = Util.findMethodAnyArgs(shadowed, "transform_swapped");
 		}
 		
-		public static void transform(Parser parser, CompilationUnitDeclaration ast) throws IOException {
+		public static void transform(Object parser, Object ast) throws IOException {
+			Main.prependClassLoader(parser.getClass().getClassLoader());
+			init(parser.getClass().getClassLoader());
 			Util.invokeMethod(TRANSFORM, parser, ast);
 		}
 		
-		public static void transform_swapped(CompilationUnitDeclaration ast, Parser parser) throws IOException {
+		public static void transform_swapped(Object ast, Object parser) throws IOException {
+			init(parser.getClass().getClassLoader());
 			Util.invokeMethod(TRANSFORM_SWAPPED, ast, parser);
 		}
 	}
@@ -190,15 +207,21 @@ final class PatchFixesHider {
 	/** Contains patch code to support {@code @Delegate} */
 	public static final class Delegate {
 		private static final Method HANDLE_DELEGATE_FOR_TYPE;
+		private static final Method GET_CHILDREN;
 		
 		static {
 			Class<?> shadowed = Util.shadowLoadClass("lombok.eclipse.agent.PatchDelegatePortal");
 			HANDLE_DELEGATE_FOR_TYPE = Util.findMethod(shadowed, "handleDelegateForType", Object.class);
+			GET_CHILDREN = Util.findMethod(shadowed, "getChildren", Object.class, Object.class);
 		}
 		
 		public static boolean handleDelegateForType(Object classScope) {
 			return (Boolean) Util.invokeMethod(HANDLE_DELEGATE_FOR_TYPE, classScope);
 		}
+		
+		public static Object[] getChildren(Object returnValue, Object javaElement) {
+			return (Object[]) Util.invokeMethod(GET_CHILDREN, returnValue, javaElement);
+		}
 	}
 	
 	/** Contains patch code to support {@code val} (eclipse specific) */
@@ -270,17 +293,19 @@ final class PatchFixesHider {
 		private static final Method RESOLVE_TYPE;
 		private static final Method ERROR_NO_METHOD_FOR;
 		private static final Method INVALID_METHOD, INVALID_METHOD2;
+		private static final Method NON_STATIC_ACCESS_TO_STATIC_METHOD;
 		
 		static {
 			Class<?> shadowed = Util.shadowLoadClass("lombok.eclipse.agent.PatchExtensionMethod");
-			RESOLVE_TYPE = Util.findMethod(shadowed, "resolveType", TypeBinding.class, MessageSend.class, BlockScope.class);
+			RESOLVE_TYPE = Util.findMethod(shadowed, "resolveType", Object.class, MessageSend.class, BlockScope.class);
 			ERROR_NO_METHOD_FOR = Util.findMethod(shadowed, "errorNoMethodFor", ProblemReporter.class, MessageSend.class, TypeBinding.class, TypeBinding[].class);
 			INVALID_METHOD = Util.findMethod(shadowed, "invalidMethod", ProblemReporter.class, MessageSend.class, MethodBinding.class);
 			INVALID_METHOD2 = Util.findMethod(shadowed, "invalidMethod", ProblemReporter.class, MessageSend.class, MethodBinding.class, Scope.class);
+			NON_STATIC_ACCESS_TO_STATIC_METHOD = Util.findMethod(shadowed, "nonStaticAccessToStaticMethod", ProblemReporter.class, ASTNode.class, MethodBinding.class, MessageSend.class);
 		}
 		
-		public static TypeBinding resolveType(TypeBinding resolvedType, MessageSend methodCall, BlockScope scope) {
-			return (TypeBinding) Util.invokeMethod(RESOLVE_TYPE, resolvedType, methodCall, scope);
+		public static Object resolveType(Object resolvedType, MessageSend methodCall, BlockScope scope) {
+			return Util.invokeMethod(RESOLVE_TYPE, resolvedType, methodCall, scope);
 		}
 		
 		public static void errorNoMethodFor(ProblemReporter problemReporter, MessageSend messageSend, TypeBinding recType, TypeBinding[] params) {
@@ -294,6 +319,30 @@ final class PatchFixesHider {
 		public static void invalidMethod(ProblemReporter problemReporter, MessageSend messageSend, MethodBinding method, Scope scope) {
 			Util.invokeMethod(INVALID_METHOD2, problemReporter, messageSend, method, scope);
 		}
+		
+		public static void nonStaticAccessToStaticMethod(ProblemReporter problemReporter, ASTNode location, MethodBinding method, MessageSend messageSend) {
+			Util.invokeMethod(NON_STATIC_ACCESS_TO_STATIC_METHOD, problemReporter, location, method, messageSend);
+		}
+	}
+	
+	/** Contains patch code to support Javadoc for generated methods */
+	public static final class Javadoc {
+		private static final Method GET_HTML;
+		private static final Method PRINT_METHOD;
+		
+		static {
+			Class<?> shadowed = Util.shadowLoadClass("lombok.eclipse.agent.PatchJavadoc");
+			GET_HTML = Util.findMethod(shadowed, "getHTMLContentFromSource", String.class, Object.class);
+			PRINT_METHOD = Util.findMethod(shadowed, "printMethod", AbstractMethodDeclaration.class, Integer.class, StringBuffer.class, TypeDeclaration.class);
+		}
+		
+		public static String getHTMLContentFromSource(String original, IJavaElement member) {
+			return (String) Util.invokeMethod(GET_HTML, original, member);
+		}
+		
+		public static StringBuffer printMethod(AbstractMethodDeclaration methodDeclaration, int tab, StringBuffer output, TypeDeclaration type) {
+			return (StringBuffer) Util.invokeMethod(PRINT_METHOD, methodDeclaration, tab, output, type);
+		}
 	}
 	
 	/**
@@ -496,6 +545,10 @@ final class PatchFixesHider {
 			return original == -1 ? end : original;
 		}
 		
+		public static int fixRetrieveStartBlockPosition(int original, int start) {
+			return original == -1 ? start : original;
+		}
+		
 		public static int fixRetrieveRightBraceOrSemiColonPosition(int original, int end) {
 //			if (original == -1) {
 //				Thread.dumpStack();
@@ -505,18 +558,24 @@ final class PatchFixesHider {
 		
 		public static int fixRetrieveRightBraceOrSemiColonPosition(int retVal, AbstractMethodDeclaration amd) {
 			if (retVal != -1 || amd == null) return retVal;
-			boolean isGenerated = EclipseAugments.ASTNode_generatedBy.get(amd) != null;
+			boolean isGenerated = ASTNode_generatedBy.get(amd) != null;
 			if (isGenerated) return amd.declarationSourceEnd;
 			return -1;
 		}
 		
 		public static int fixRetrieveRightBraceOrSemiColonPosition(int retVal, FieldDeclaration fd) {
 			if (retVal != -1 || fd == null) return retVal;
-			boolean isGenerated = EclipseAugments.ASTNode_generatedBy.get(fd) != null;
+			boolean isGenerated = ASTNode_generatedBy.get(fd) != null;
 			if (isGenerated) return fd.declarationSourceEnd;
 			return -1;
 		}
 		
+		public static int fixRetrieveProperRightBracketPosition(int retVal, Type type) {
+			if (retVal != -1 || type == null) return retVal;
+			if (isGenerated(type)) return type.getStartPosition() + type.getLength() - 1;
+			return -1;
+		}
+		
 		public static final int ALREADY_PROCESSED_FLAG = 0x800000;  //Bit 24
 		
 		public static boolean checkBit24(Object node) throws Exception {
@@ -532,13 +591,13 @@ final class PatchFixesHider {
 				org.eclipse.jdt.internal.compiler.ast.ASTNode internalNode) throws Exception {
 			
 			if (internalNode == null || domNode == null) return;
-			boolean isGenerated = EclipseAugments.ASTNode_generatedBy.get(internalNode) != null;
+			boolean isGenerated = ASTNode_generatedBy.get(internalNode) != null;
 			if (isGenerated) domNode.getClass().getField("$isGenerated").set(domNode, true);
 		}
 		
 		public static void setIsGeneratedFlagForName(org.eclipse.jdt.core.dom.Name name, Object internalNode) throws Exception {
 			if (internalNode instanceof org.eclipse.jdt.internal.compiler.ast.ASTNode) {
-				boolean isGenerated = EclipseAugments.ASTNode_generatedBy.get((org.eclipse.jdt.internal.compiler.ast.ASTNode) internalNode) != null;
+				boolean isGenerated = ASTNode_generatedBy.get((org.eclipse.jdt.internal.compiler.ast.ASTNode) internalNode) != null;
 				if (isGenerated) name.getClass().getField("$isGenerated").set(name, true);
 			}
 		}
