@@ -1,3 +1,6 @@
+//skip-idempotent
+import lombok.Builder;
+@Builder
 public class BuilderDefaultsWarnings {
 	long x = System.currentTimeMillis();
 	final int y = 5;
@@ -20,11 +23,17 @@ public class BuilderDefaultsWarnings {
 		@java.lang.SuppressWarnings("all")
 		BuilderDefaultsWarningsBuilder() {
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public BuilderDefaultsWarnings.BuilderDefaultsWarningsBuilder x(final long x) {
 			this.x = x;
 			return this;
 		}
+		/**
+		 * @return {@code this}.
+		 */
 		@java.lang.SuppressWarnings("all")
 		public BuilderDefaultsWarnings.BuilderDefaultsWarningsBuilder z(final int z) {
 			this.z = z;
@@ -78,6 +87,7 @@ public class BuilderDefaultsWarnings {
 }
 class NoBuilderButHasDefaults {
 	private final long z = 5;
+	@Builder
 	public NoBuilderButHasDefaults() {
 	}
 	@java.lang.SuppressWarnings("all")
