@@ -1,4 +1,6 @@
+//skip-idempotent
 import java.util.List;
+@lombok.Builder
 class BuilderSingularNoAuto {
 	private List<String> things;
 	private List<String> widgets;
