@@ -10,3 +10,8 @@ class LoggerJulWithDifferentName {
 	@java.lang.SuppressWarnings("all")
 	private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger("DifferentName");
 }
+class LoggerJulWithStaticField {
+	@java.lang.SuppressWarnings("all")
+	private static final java.util.logging.Logger log = java.util.logging.Logger.getLogger(LoggerJulWithStaticField.TOPIC);
+	static final String TOPIC = "StaticField";
+}
\ No newline at end of file
