@@ -10,3 +10,8 @@ class LoggerLog4j2WithDifferentName {
 	@java.lang.SuppressWarnings("all")
 	private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger("DifferentName");
 }
+class LoggerLog4j2WithStaticField {
+	@java.lang.SuppressWarnings("all")
+	private static final org.apache.logging.log4j.Logger log = org.apache.logging.log4j.LogManager.getLogger(LoggerLog4j2WithStaticField.TOPIC);
+	static final String TOPIC = "StaticField";
+}
\ No newline at end of file
