@@ -35,4 +35,13 @@ class LoggerJBossLogOuter {
   LoggerJBossLogWithDifferentLoggerName() {
     super();
   }
+}
+@JBossLog(topic = LoggerJBossLogWithStaticField.TOPIC) class LoggerJBossLogWithStaticField {
+  private static final org.jboss.logging.Logger log = org.jboss.logging.Logger.getLogger(LoggerJBossLogWithStaticField.TOPIC);
+  static final String TOPIC = "StaticField";
+  <clinit>() {
+  }
+  LoggerJBossLogWithStaticField() {
+    super();
+  }
 }
\ No newline at end of file
