@@ -1,9 +1,10 @@
-//CONF: checkerframework = 3.0
+//CONF: checkerframework = 4.0
+import lombok.AllArgsConstructor;
 import lombok.Data;
 import lombok.experimental.Accessors;
 import lombok.With;
 
-@Data @Accessors(chain = true)
+@Data @AllArgsConstructor @Accessors(chain = true)
 class CheckerFrameworkBasic {
 	@With private final int x;
 	private final int y;
