@@ -17,4 +17,9 @@ class LoggerJBossLogOuter {
 
 @JBossLog(topic="DifferentLogger")
 class LoggerJBossLogWithDifferentLoggerName {
+}
+
+@JBossLog(topic=LoggerJBossLogWithStaticField.TOPIC)
+class LoggerJBossLogWithStaticField {
+	static final String TOPIC = "StaticField";
 }
\ No newline at end of file
